@@ -8,11 +8,14 @@ sys.path.append("@FENCEAGENTSLIBDIR@")
 from fencing import *
 from fencing import fail, run_delay, EC_LOGIN_DENIED, EC_STATUS
 
+if sys.version_info[0] > 2: import urllib.parse as urllib
+else: import urllib
+
 state = {"POWERED_ON": "on", 'POWERED_OFF': "off", 'SUSPENDED': "off"}
 
 def get_power_status(conn, options):
 	try:
-		res = send_command(conn, "vcenter/vm?filter.names={}".format(options["--plug"]))["value"]
+		res = send_command(conn, "vcenter/vm?filter.names={}".format(urllib.quote(options["--plug"])))["value"]
 	except Exception as e:
 		logging.debug("Failed: {}".format(e))
 		fail(EC_STATUS)
@@ -42,10 +45,20 @@ def get_list(conn, options):
 	outlets = {}
 
 	try:
-		res = send_command(conn, "vcenter/vm")
-	except:
+		command = "vcenter/vm"
+		if "--filter" in options:
+			command = command + "?" + options["--filter"]
+		res = send_command(conn, command)
+	except Exception as e:
 		logging.debug("Failed: {}".format(e))
-		fail(EC_STATUS)
+		if str(e).startswith("400"):
+			if options.get("--original-action") == "monitor":
+				return outlets
+			else:
+				logging.error("More than 1000 VMs returned. Use --filter parameter to limit which VMs to list.")
+				fail(EC_STATUS)
+		else:
+			fail(EC_STATUS)
 
 	for r in res["value"]:
 		outlets[r["name"]] = ("", state[r["power_state"]])
@@ -56,7 +69,7 @@ def connect(opt):
 	conn = pycurl.Curl()
 
 	## setup correct URL
-	if "--ssl" in opt or "--ssl-secure" in opt or "--ssl-insecure" in opt:
+	if "--ssl-secure" in opt or "--ssl-insecure" in opt:
 		conn.base_url = "https:"
 	else:
 		conn.base_url = "http:"
@@ -76,11 +89,11 @@ def connect(opt):
 	conn.setopt(pycurl.USERPWD, opt["--username"] + ":" + opt["--password"])
 
 	conn.setopt(pycurl.TIMEOUT, int(opt["--shell-timeout"]))
-	if "--ssl" in opt or "--ssl-secure" in opt:
+
+	if "--ssl-secure" in opt:
 		conn.setopt(pycurl.SSL_VERIFYPEER, 1)
 		conn.setopt(pycurl.SSL_VERIFYHOST, 2)
-
-	if "--ssl-insecure" in opt:
+	elif "--ssl-insecure" in opt:
 		conn.setopt(pycurl.SSL_VERIFYPEER, 0)
 		conn.setopt(pycurl.SSL_VERIFYHOST, 0)
 
@@ -124,10 +137,10 @@ def send_command(conn, command, method="GET"):
 	try:
 		conn.perform()
 	except Exception as e:
-		raise Exception(e[1])
+		raise(e)
 
 	rc = conn.getinfo(pycurl.HTTP_CODE)
-	result = web_buffer.getvalue().decode()
+	result = web_buffer.getvalue().decode("UTF-8")
 
 	web_buffer.close()
 
@@ -135,7 +148,11 @@ def send_command(conn, command, method="GET"):
 		result = json.loads(result)
 
 	if rc != 200:
-		raise Exception("{}: {}".format(rc, result["value"]["messages"][0]["default_message"]))
+		if len(result) > 0:
+			raise Exception("{}: {}".format(rc, 
+					result["value"]["messages"][0]["default_message"]))
+		else:
+			raise Exception("Remote returned {} for request to {}".format(rc, url))
 
 	logging.debug("url: {}".format(url))
 	logging.debug("method: {}".format(method))
@@ -153,6 +170,15 @@ def define_new_opts():
 		"required" : "0",
 		"shortdesc" : "The path part of the API URL",
 		"order" : 2}
+	all_opt["filter"] = {
+		"getopt" : ":",
+		"longopt" : "filter",
+		"help" : "--filter=[filter]              Filter to only return relevant VMs"
+			 " (e.g. \"filter.names=node1&filter.names=node2\").",
+		"required" : "0",
+		"shortdesc" : "Filter to only return relevant VMs. It can be used to avoid "
+			      "the agent failing when more than 1000 VMs should be returned.",
+		"order" : 2}
 
 
 def main():
@@ -165,6 +191,7 @@ def main():
 		"notls",
 		"web",
 		"port",
+		"filter",
 	]
 
 	atexit.register(atexit_handler)
@@ -177,8 +204,12 @@ def main():
 
 	docs = {}
 	docs["shortdesc"] = "Fence agent for VMware REST API"
-	docs["longdesc"] = "fence_vmware_rest is an I/O Fencing agent which can be \
-used with VMware API to fence virtual machines."
+	docs["longdesc"] = """fence_vmware_rest is an I/O Fencing agent which can be \
+used with VMware API to fence virtual machines.
+
+NOTE: If there's more than 1000 VMs there is a filter parameter to work around \
+the API limit. See https://code.vmware.com/apis/62/vcenter-management#/VM%20/get_vcenter_vm \
+for full list of filters."""
 	docs["vendorurl"] = "https://www.vmware.com"
 	show_docs(options, docs)
 
