@@ -26,7 +26,7 @@ def open_socket(options):
 	except socket.gaierror:
 		fail(EC_LOGIN_DENIED)
 
-	if "--ssl" in options:
+	if "--ssl-secure" in options or "--ssl-insecure" in options:
 		import ssl
 		sock = socket.socket()
 		sslcx = ssl.create_default_context()
@@ -127,12 +127,17 @@ def get_list_of_images(options, command, data_as_plug):
 
 	conn.send(packet)
 
-	request_id = struct.unpack("!i", conn.recv(INT4))[0]
-	(output_len, request_id, return_code, reason_code) = struct.unpack("!iiii", conn.recv(INT4 * 4))
+	try:
+		request_id = struct.unpack("!i", conn.recv(INT4))[0]
+		(output_len, request_id, return_code, reason_code) = struct.unpack("!iiii", conn.recv(INT4 * 4))
+	except struct.error:
+		logging.debug(sys.exc_info())
+		fail_usage("Failed: Unable to connect to {} port: {} SSL: {} \n".format(options["--ip"], options["--ipport"], bool("--ssl" in options)))
+
 	images = set()
 
 	if output_len > 3*INT4:
-		recvflag = socket.MSG_WAITALL if "--ssl" not in options else 0
+		recvflag = socket.MSG_WAITALL if "--ssl-secure" not in options and "--ssl-insecure" not in options else 0
 		array_len = struct.unpack("!i", conn.recv(INT4))[0]
 		data = ""
 
@@ -156,11 +161,24 @@ def get_list_of_images(options, command, data_as_plug):
 	conn.close()
 	return (return_code, reason_code, images)
 
+def define_new_opts():
+	all_opt["disable_ssl"] = {
+		"getopt" : "",
+		"longopt" : "disable-ssl",
+		"help" : "--disable-ssl                  Don't use SSL connection",
+		"required" : "0",
+		"shortdesc" : "Don't use SSL",
+		"order": 2
+	}
+
 def main():
 	device_opt = ["ipaddr", "login", "passwd", "port", "method", "missing_as_off",
-		      "inet4_only", "inet6_only", "ssl"]
+		      "inet4_only", "inet6_only", "ssl", "disable_ssl"]
 
 	atexit.register(atexit_handler)
+	define_new_opts()
+
+	all_opt["ssl"]["help"] = "-z, --ssl                      Use SSL connection with verifying certificate (Default)"
 
 	all_opt["ipport"]["default"] = "44444"
 	all_opt["shell_timeout"]["default"] = "5"
@@ -168,6 +186,11 @@ def main():
 	all_opt["ssl"]["default"] = "1"
 	options = check_input(device_opt, process_input(device_opt), other_conditions=True)
 
+	if "--disable-ssl" in options or options["--ssl"] == "0":
+		for k in ["--ssl", "--ssl-secure", "--ssl-insecure"]:
+			if k in options:
+				del options[k]
+
 	if len(options.get("--plug", "")) > 8:
 		fail_usage("Failed: Name of image can not be longer than 8 characters")
 
